@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2013 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,13 +18,24 @@
 Main function module for link checking.
 """
 
-# imports and checks
+# version checks
 import sys
 # Needs Python >= 2.7 because we use dictionary based logging config
 # Needs Python >= 2.7.2 which fixed http://bugs.python.org/issue11467
 if not (hasattr(sys, 'version_info') or
         sys.version_info < (2, 7, 2, 'final', 0)):
-    raise SystemExit("This program requires Python 2.7.2 or later.")
+    import platform
+    version = platform.python_version()
+    raise SystemExit("This program requires Python 2.7.2 or later instead of %s." % version)
+# require a reasonably recent requests module: 2.4.0 from 2014-08-29
+import requests
+# PEP 396 has only version strings, bummer! PEP 386 is also not helpful.
+requests_version = requests.__version__.split('.')
+# Depends on the version scheme of Python requests
+if int(requests_version[0]) < 2 or \
+   (int(requests_version[0]) == 2 and int(requests_version[1]) < 4):
+    raise SystemExit("This program requires Python requests 2.4.0 or later instead of %s." % requests.__version__)
+
 import os
 # add the custom linkcheck_dns directory to sys.path
 _dnspath = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'linkcheck_dns'))
@@ -35,15 +46,19 @@ import re
 import signal
 import traceback
 
-from . import i18n
+from . import i18n, log
+from .logconf import (
+    LOG_ROOT,
+    LOG_CMDLINE,
+    LOG_CHECK,
+    LOG_CACHE,
+    LOG_GUI,
+    LOG_THREAD,
+    LOG_PLUGIN,
+)
 import _LinkChecker_configdata as configdata
 
 
-def main_is_frozen ():
-    """Return True iff running inside a py2exe-generated executable."""
-    return hasattr(sys, "frozen")
-
-
 def module_path ():
     """Return absolute directory of system executable."""
     return os.path.dirname(os.path.abspath(sys.executable))
@@ -51,55 +66,12 @@ def module_path ():
 
 def get_install_data ():
     """Return absolute path of LinkChecker data installation directory."""
-    if main_is_frozen():
+    from .loader import is_frozen
+    if is_frozen():
         return module_path()
     return configdata.install_data
 
 
-def get_config_dir ():
-    """Return absolute path of LinkChecker example configuration."""
-    return os.path.join(get_install_data(), "share", "linkchecker")
-
-
-# application log areas
-LOG_ROOT = "linkcheck"
-LOG_CMDLINE = "linkcheck.cmdline"
-LOG_CHECK = "linkcheck.check"
-LOG_CACHE = "linkcheck.cache"
-LOG_GUI = "linkcheck.gui"
-LOG_THREAD = "linkcheck.thread"
-lognames = {
-    "cmdline": LOG_CMDLINE,
-    "checking": LOG_CHECK,
-    "cache": LOG_CACHE,
-    "gui": LOG_GUI,
-    "thread": LOG_THREAD,
-    "all": LOG_ROOT,
-}
-lognamelist = ", ".join(repr(name) for name in lognames)
-
-# logging configuration
-configdict = {
-    'version': 1,
-    'loggers': {
-    },
-    'root': {
-      'level': 'DEBUG',
-    },
-}
-
-def init_log_config():
-    """Configure the application loggers."""
-    for applog in lognames.values():
-        # propagate except for root app logger 'linkcheck'
-        propagate = (applog != LOG_ROOT)
-        configdict['loggers'][applog] = dict(level='INFO', propagate=propagate)
-
-init_log_config()
-
-from . import log
-
-
 class LinkCheckerError (StandardError):
     """Exception to be raised on linkchecker-specific check errors."""
     pass
@@ -119,6 +91,7 @@ def get_link_pat (arg, strict=False):
     @type strict: bool
     @return: dictionary with keys 'pattern', 'negate' and 'strict'
     @rtype: dict
+    @raises: re.error on invalid regular expressions
     """
     log.debug(LOG_CHECK, "Link pattern %r strict=%s", arg, strict)
     if arg.startswith('!'):
@@ -127,8 +100,13 @@ def get_link_pat (arg, strict=False):
     else:
         pattern = arg
         negate = False
+    try:
+        regex = re.compile(pattern)
+    except re.error as msg:
+        log.warn(LOG_CHECK, "invalid regular expression %r: %s" % (pattern, msg))
+        raise
     return {
-        "pattern": re.compile(pattern),
+        "pattern": regex,
         "negate": negate,
         "strict": strict,
     }
@@ -155,6 +133,7 @@ def init_i18n (loc=None):
     logging.addLevelName(logging.DEBUG, _('DEBUG'))
     logging.addLevelName(logging.NOTSET, _('NOTSET'))
 
+
 # initialize i18n, puts _() and _n() function into global namespace
 init_i18n()
 
