@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
-ANSI Color definitions and functions. For Windows systems, WConio is
-required.
+ANSI Color definitions and functions. For Windows systems, the colorama module
+uses ctypes and Windows DLLs to generate colored output.
 
 From Term::ANSIColor, applies also to this module:
 
@@ -54,16 +54,15 @@ ISO 6429 is available from ISO for a charge; the author of this module does
 not own a copy of it. Since the source material for ISO 6429 was ECMA-048
 and the latter is available for free, there seems little reason to obtain
 the ISO standard.
-
-WConio module: http://newcenturycomputers.net/projects/wconio.html
 """
 
 import os
 import logging
 import types
 from .fileutil import has_module, is_tty
+if os.name == 'nt':
+    from . import colorama
 
-has_wconio = has_module("WConio")
 has_curses = has_module("curses")
 
 # Color constants
@@ -114,8 +113,9 @@ Purple = 'Purple'
 Cyan = 'Cyna'
 White = 'White'
 
+InverseColors = (Black, Red, Green, Yellow, Blue, Purple, Cyan, White)
 
-# Color numbers; capitalized colors are inverse
+# Ansi color numbers; capitalized colors are inverse
 AnsiColor = {
     None:    '0',
     default: '0',
@@ -137,6 +137,29 @@ AnsiColor = {
     White:   '47',
 }
 
+if os.name == 'nt':
+    # Windows color numbers; capitalized colors are used as background
+    WinColor = {
+        None:    None,
+        default: colorama.GREY,
+        black:   colorama.BLACK,
+        red:     colorama.RED,
+        green:   colorama.GREEN,
+        yellow:  colorama.YELLOW,
+        blue:    colorama.BLUE,
+        purple:  colorama.MAGENTA,
+        cyan:    colorama.CYAN,
+        white:   colorama.GREY,
+        Black:   colorama.BLACK,
+        Red:     colorama.RED,
+        Green:   colorama.GREEN,
+        Yellow:  colorama.YELLOW,
+        Blue:    colorama.BLUE,
+        Purple:  colorama.MAGENTA,
+        Cyan:    colorama.CYAN,
+        White:   colorama.GREY,
+    }
+
 # pc speaker beep escape code
 Beep = "\007"
 
@@ -153,40 +176,64 @@ def esc_ansicolor (color):
 AnsiReset = esc_ansicolor(default)
 
 
+def get_win_color(color):
+    """Convert a named color definition to Windows console color foreground,
+    background and style numbers."""
+    foreground = background = style = None
+    control = ''
+    if ";" in color:
+        control, color = color.split(";", 1)
+        if control == bold:
+            style = colorama.BRIGHT
+    if color in InverseColors:
+        background = WinColor[color]
+    else:
+        foreground = WinColor.get(color)
+    return foreground, background, style
+
+
 def has_colors (fp):
     """Test if given file is an ANSI color enabled tty."""
-    # The isatty() function ensures that we do not colorize
+    # The is_tty() function ensures that we do not colorize
     # redirected streams, as this is almost never what we want
     if not is_tty(fp):
         return False
     if os.name == 'nt':
-        # On Win9x system, ANSI.SYS would also work. But this
-        # requires manually adding it to config.sys, which is
-        # unlikely someone will do just to run this software on
-        # an old system.
-        return has_wconio
+        return True
     elif has_curses:
         import curses
         try:
-            curses.setupterm()
-            if curses.tigetnum("colors") >= 8:
-                # We have at least 8 colors - allright!
-                return True
+            curses.setupterm(os.environ.get("TERM"), fp.fileno())
+            # More than 8 colors are good enough.
+            return curses.tigetnum("colors") >= 8
         except curses.error:
-            # initialize error, eg. no terminfo database found
             return False
     return False
 
 
-def _write_color_nt (fp, text, color):
-    """Assumes WConio has been imported at module level."""
-    oldcolor = WConio.gettextinfo()[4]
-    oldtextcolor = oldcolor & 0x000F
-    if ";" in color:
-        color = color.split(";", 1)[1]
-    WConio.textcolor(WConioColor.get(color, oldtextcolor))
+def get_columns (fp):
+    """Return number of columns for given file."""
+    if not is_tty(fp):
+        return 80
+    if os.name == 'nt':
+        return colorama.get_console_size().X
+    if has_curses:
+        import curses
+        try:
+            curses.setupterm(os.environ.get("TERM"), fp.fileno())
+            return curses.tigetnum("cols")
+        except curses.error:
+           pass
+    return 80
+
+
+def _write_color_colorama (fp, text, color):
+    """Colorize text with given color."""
+    foreground, background, style = get_win_color(color)
+    colorama.set_console(foreground=foreground, background=background,
+      style=style)
     fp.write(text)
-    WConio.textattr(oldcolor)
+    colorama.reset_console()
 
 
 def _write_color_ansi (fp, text, color):
@@ -196,29 +243,9 @@ def _write_color_ansi (fp, text, color):
     fp.write(AnsiReset)
 
 
-if os.name == 'nt' and has_wconio:
-    import WConio
-    WConioColor = {
-        None: WConio.LIGHTGREY,
-        default: WConio.LIGHTGREY,
-        black: WConio.BLACK,
-        red: WConio.RED,
-        green: WConio.GREEN,
-        yellow: WConio.YELLOW,
-        blue: WConio.BLUE,
-        purple: WConio.MAGENTA,
-        cyan: WConio.CYAN,
-        white: WConio.WHITE,
-        Black: WConio.BLACK,
-        Red: WConio.RED,
-        Green: WConio.GREEN,
-        Yellow: WConio.YELLOW,
-        Blue: WConio.BLUE,
-        Purple: WConio.MAGENTA,
-        Cyan: WConio.CYAN,
-        White: WConio.WHITE,
-    }
-    write_color = _write_color_nt
+if os.name == 'nt':
+    write_color = _write_color_colorama
+    colorama.init()
 else:
     write_color = _write_color_ansi
 
