@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -72,14 +72,14 @@ class NntpUrl (urlbase.UrlBase):
         later", are caught.
         """
         tries = 0
-        nntp = value = None
+        nntp = None
         while tries < 2:
             tries += 1
             try:
                 nntp = nntplib.NNTP(nntpserver, usenetrc=False)
             except nntplib.NNTPTemporaryError:
                 self.wait()
-            except nntplib.NNTPPermanentError, msg:
+            except nntplib.NNTPPermanentError as msg:
                 if re.compile("^50[45]").search(str(msg)):
                     self.wait()
                 else:
