@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2010-2012 Bastian Kleineidam
+# Copyright (C) 2010-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -85,10 +85,10 @@ class Settings (object):
 
     def read_options (self):
         """Return stored GUI options."""
-        data = dict(debug=None, debugmemory=None, verbose=None,
+        data = dict(debug=None, verbose=None,
             recursionlevel=None, warninglines=None, ignorelines=None)
         self.settings.beginGroup('output')
-        for key in ("debug", "debugmemory", "verbose"):
+        for key in ("debug", "verbose"):
             if self.settings.contains(key):
                 data[key] = self.settings.value(key).toBool()
         self.settings.endGroup()
@@ -116,7 +116,7 @@ class Settings (object):
     def save_options (self, data):
         """Save GUI options."""
         self.settings.beginGroup('output')
-        for key in ("debug", "debugmemory", "verbose"):
+        for key in ("debug", "verbose"):
             self.settings.setValue(key, QtCore.QVariant(data[key]))
         self.settings.endGroup()
         self.settings.beginGroup('checking')
@@ -137,6 +137,23 @@ class Settings (object):
         self.settings.setValue('documents', QtCore.QVariant(data))
         self.settings.endGroup()
 
+    def read_misc(self):
+        """Get misc options."""
+        data = dict(saveresultas=None)
+        self.settings.beginGroup('misc')
+        key = 'saveresultas'
+        value = self.settings.value(key).toString()
+        data[key] = unicode(value)
+        self.settings.endGroup()
+        return data
+
+    def save_misc(self, data):
+        """Save misc options."""
+        self.settings.beginGroup('misc')
+        key = 'saveresultas'
+        self.settings.setValue(key, QtCore.QVariant(data[key]))
+        self.settings.endGroup()
+
     def sync (self):
         """Synchronize QSettings object to disk."""
         self.settings.sync()
