@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2012 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,66 +19,16 @@ Functions for parsing and matching URL strings.
 """
 
 import re
-import sys
 import os
 import urlparse
 import urllib
-import urllib2
-import socket
-from . import httplib2 as httplib
+import requests
 from . import log, LOG_CHECK
 
 for scheme in ('ldap', 'irc'):
     if scheme not in urlparse.uses_netloc:
         urlparse.uses_netloc.append(scheme)
 
-if sys.version_info[0] > 2 or sys.version_info[1] > 6:
-    # Fix Python regression; see http://bugs.python.org/issue11467
-    def urlsplit_26(url, scheme='', allow_fragments=True):
-        """Parse a URL into 5 components:
-        <scheme>://<netloc>/<path>?<query>#<fragment>
-        Return a 5-tuple: (scheme, netloc, path, query, fragment).
-        Note that we don't break the components up in smaller bits
-        (e.g. netloc is a single string) and we don't expand % escapes."""
-        allow_fragments = bool(allow_fragments)
-        key = url, scheme, allow_fragments, type(url), type(scheme)
-        cached = urlparse._parse_cache.get(key, None)
-        if cached:
-            return cached
-        if len(urlparse._parse_cache) >= urlparse.MAX_CACHE_SIZE: # avoid runaway growth
-            urlparse.clear_cache()
-        netloc = query = fragment = ''
-        i = url.find(':')
-        if i > 0:
-            if url[:i] == 'http': # optimize the common case
-                scheme = url[:i].lower()
-                url = url[i+1:]
-                if url[:2] == '//':
-                    netloc, url = urlparse._splitnetloc(url, 2)
-                if allow_fragments and '#' in url:
-                    url, fragment = url.split('#', 1)
-                if '?' in url:
-                    url, query = url.split('?', 1)
-                v = urlparse.SplitResult(scheme, netloc, url, query, fragment)
-                urlparse._parse_cache[key] = v
-                return v
-            for c in url[:i]:
-                if c not in urlparse.scheme_chars:
-                    break
-            else:
-                scheme, url = url[:i].lower(), url[i+1:]
-        if url[:2] == '//':
-            netloc, url = urlparse._splitnetloc(url, 2)
-        if allow_fragments and scheme in urlparse.uses_fragment and '#' in url:
-            url, fragment = url.split('#', 1)
-        if scheme in urlparse.uses_query and '?' in url:
-            url, query = url.split('?', 1)
-        v = urlparse.SplitResult(scheme, netloc, url, query, fragment)
-        urlparse._parse_cache[key] = v
-        return v
-
-    urlparse.urlsplit = urlsplit_26
-
 # The character set to encode non-ASCII characters in a URL. See also
 # http://tools.ietf.org/html/rfc2396#section-2.1
 # Note that the encoding is not really specified, but most browsers
@@ -98,6 +48,7 @@ default_ports = {
     'http': 80,
     'https': 443,
     'nntps': 563,
+    'ftp': 21,
 }
 
 # adapted from David Wheelers "Secure Programming for Linux and Unix HOWTO"
@@ -317,6 +268,19 @@ def url_parse_query (query, encoding=None):
     return ''.join(l) + append
 
 
+def urlunsplit (urlparts):
+    """Same as urlparse.urlunsplit but with extra UNC path handling
+    for Windows OS."""
+    res = urlparse.urlunsplit(urlparts)
+    if os.name == 'nt' and urlparts[0] == 'file' and '|' not in urlparts[2]:
+        # UNC paths must have 4 slashes: 'file:////server/path'
+        # Depending on the path in urlparts[2], urlparse.urlunsplit()
+        # left only two or three slashes. This is fixed below
+        repl = 'file://' if urlparts[2].startswith('//') else 'file:/'
+        res = res.replace('file:', repl)
+    return res
+
+
 def url_norm (url, encoding=None):
     """Normalize the given URL which must be quoted. Supports unicode
     hostnames (IDNA encoding) according to RFC 3490.
@@ -344,8 +308,7 @@ def url_norm (url, encoding=None):
     is_idn = url_fix_host(urlparts)
     # query
     urlparts[3] = url_parse_query(urlparts[3], encoding=encoding)
-    is_hierarchical = urlparts[0] not in urlparse.non_hierarchical
-    if is_hierarchical:
+    if urlparts[0] in urlparse.uses_relative:
         # URL has a hierarchical path we should norm
         if not urlparts[2]:
             # Empty path is allowed if both query and fragment are also empty.
@@ -363,7 +326,7 @@ def url_norm (url, encoding=None):
     urlparts[1] = url_quote_part(urlparts[1], safechars='@:', encoding=encoding) # host
     urlparts[2] = url_quote_part(urlparts[2], safechars=_nopathquote_chars, encoding=encoding) # path
     urlparts[4] = url_quote_part(urlparts[4], encoding=encoding) # anchor
-    res = urlparse.urlunsplit(urlparts)
+    res = urlunsplit(urlparts)
     if url.endswith('#') and not urlparts[4]:
         # re-append trailing empty fragment
         res += '#'
@@ -432,7 +395,7 @@ def url_quote (url):
             l.append("%s%s" % (k, sep))
     urlparts[3] = ''.join(l)
     urlparts[4] = url_quote_part(urlparts[4]) # anchor
-    return urlparse.urlunsplit(urlparts)
+    return urlunsplit(urlparts)
 
 
 def url_quote_part (s, safechars='/', encoding=None):
@@ -499,7 +462,7 @@ def url_split (url):
     """
     scheme, netloc = urllib.splittype(url)
     host, document = urllib.splithost(netloc)
-    port = default_ports.get(scheme, 80)
+    port = default_ports.get(scheme, 0)
     if host:
         host = host.lower()
         host, port = splitport(host, port=port)
@@ -513,108 +476,93 @@ def url_unsplit (parts):
     return "%s://%s:%d%s" % parts
 
 
-def splitport (host, port=80):
+def splitport (host, port=0):
     """Split optional port number from host. If host has no port number,
     the given default port is returned.
 
     @param host: host name
     @ptype host: string
+    @param port: the port number (default 0)
+    @ptype port: int
+
     @return: tuple of (host, port)
     @rtype: tuple of (string, int)
     """
     if ":" in host:
-        host, sport = host.split(":", 1)
+        shost, sport = host.split(":", 1)
         iport = is_numeric_port(sport)
         if iport:
-            port = iport
+            host, port = shost, iport
+        elif not sport:
+            # empty port, ie. the host was "hostname:"
+            host = shost
+        else:
+            # For an invalid non-empty port leave the host name as is
+            pass
     return host, port
 
 
-class PasswordManager (object):
-    """Simple password manager storing username and password. Suitable
-    for use as an AuthHandler instance in urllib2."""
-
-    def __init__ (self, user, password):
-        """Store given username and password."""
-        self.user = user
-        self.password = password
-
-    def add_password (self, realm, uri, user, passwd):
-        """Does nothing since username and password are already stored.
-
-        @return: None
-        """
-        pass
-
-    def find_user_password (self, realm, authuri):
-        """Get stored username and password.
-
-        @return: A tuple (user, password)
-        @rtype: tuple
-        """
-        return self.user, self.password
-
-
-def get_opener (user=None, password=None, proxy=None, debuglevel=0):
-    """Construct an URL opener object. It considers the given credentials
-    and proxy.
-
-    @return: URL opener
-    @rtype: urllib2.OpenerDirector
-    """
-    from . import httputil
-    pwd_manager = PasswordManager(user, password)
-    handlers = [
-        urllib2.UnknownHandler,
-        httputil.HttpWithGzipHandler(debuglevel=debuglevel),
-        urllib2.HTTPBasicAuthHandler(pwd_manager),
-        urllib2.HTTPDigestAuthHandler(pwd_manager),
-    ]
-    if proxy:
-        handlers.insert(0,
-          urllib2.ProxyHandler({"http": proxy, "https": proxy}))
-        handlers.extend([
-            urllib2.ProxyBasicAuthHandler(pwd_manager),
-            urllib2.ProxyDigestAuthHandler(pwd_manager),
-        ])
-    if hasattr(httplib, 'HTTPS'):
-        handlers.append(httputil.HttpsWithGzipHandler(debuglevel=debuglevel))
-    return urllib2.build_opener(*handlers)
-
-
-def get_content (url, user=None, password=None, proxy=None, data=None,
-                 addheaders=None):
+def get_content(url, user=None, password=None, proxy=None, data=None,
+                addheaders=None):
     """Get URL content and info.
 
-    @return: (url info, content), or (None, None) on error.
-    @rtype: tuple (string, string)
+    @return: (decoded text content of URL, headers) or
+             (None, errmsg) on error.
+    @rtype: tuple (String, dict) or (None, String)
     """
     from . import configuration
-    if log.is_debug(LOG_CHECK):
-        debuglevel = 1
-    else:
-        debuglevel = 0
     headers = {
         'User-Agent': configuration.UserAgent,
-        # makes problems with some sites
-        #'Accept-Encoding': 'gzip;q=1.0, deflate;q=0.9, identity;q=0.5',
     }
     if addheaders:
         headers.update(addheaders)
-    req = urllib2.Request(url, data, headers)
+    method = 'GET'
+    kwargs = dict(headers=headers)
+    if user and password:
+        kwargs['auth'] = (user, password)
+    if data:
+        kwargs['data'] = data
+        method = 'POST'
+    if proxy:
+        kwargs['proxy'] = dict(http=proxy)
+    from .configuration import get_share_file
     try:
-        f = get_opener(user=user, password=password, proxy=proxy,
-                       debuglevel=debuglevel)
-        res = None
-        try:
-            res = f.open(req)
-            return (res.info(), res.read())
-        finally:
-            if res is not None:
-                res.close()
-    except (urllib2.HTTPError, socket.timeout, urllib2.URLError,
-            socket.gaierror, socket.error, IOError, httplib.HTTPException,
-            ValueError), msg:
+        kwargs["verify"] = get_share_file('cacert.pem')
+    except ValueError:
+        pass
+    try:
+        response = requests.request(method, url, **kwargs)
+        return response.text, response.headers
+    except (requests.exceptions.RequestException,
+            requests.exceptions.BaseHTTPError) as msg:
         log.warn(LOG_CHECK, ("Could not get content of URL %(url)s: %(msg)s.") \
           % {"url": url, "msg": str(msg)})
-    return (None, None)
+        return None, str(msg)
+
+
+def shorten_duplicate_content_url(url):
+    """Remove anchor part and trailing index.html from URL."""
+    if '#' in url:
+        url = url.split('#', 1)[0]
+    if url.endswith('index.html'):
+        return url[:-10]
+    if url.endswith('index.htm'):
+        return url[:-9]
+    return url
+
+
+def is_duplicate_content_url(url1, url2):
+    """Check if both URLs are allowed to point to the same content."""
+    if url1 == url2:
+        return True
+    if url2 in url1:
+        url1 = shorten_duplicate_content_url(url1)
+        if not url2.endswith('/') and url1.endswith('/'):
+            url2 += '/'
+        return url1 == url2
+    if url1 in url2:
+        url2 = shorten_duplicate_content_url(url2)
+        if not url1.endswith('/') and url2.endswith('/'):
+            url1 += '/'
+        return url1 == url2
+    return False
